'use strict';

const merge = require('ember-cli-lodash-subset').merge;

module.exports = function foundation(options) {
  let defaults = {
    // @import 模块导入规则
    import: {
      path: ['node_modules/@choiceform/ui-foundation/addon/styles'],
    },

    // 保留原生 CSS 变量（高级浏览器支持）
    preserve: false,

    // CSS 变量
    variables: {
      // Primary color
      '--primary': '#2196F3',
      '--primary-darker': '#0079C4',

      // Secondary color
      '--secondary': '#4ECE3D',
      '--secondary-darker': '#40B630',

      // Accent color
      '--accent': '#373277',

      // Bright theme palette
      '--palette-bright-01': '#A8C6DF',
      '--palette-bright-02': '#C5D9E8',
      '--palette-bright-03': '#ECF5FD',
      '--palette-bright-04': '#FAFBFC',

      // Dark theme palette
      '--palette-dark-01': '#212F40',
      '--palette-dark-02': '#2C405A',
      '--palette-dark-03': '#3F536E',
      '--palette-dark-04': '#8DABC4',

      // Dark text on light backgrounds
      // Primary text
      '--primary-text-dark': '#3F536E',
      // Secondary text
      '--secondary-text-dark': '#8DABC4',
      // Disabled text, hint text, and icons
      '--disabled-text-dark': '#A8C6DF',
      // Dividers
      '--dividers-dark': '#C5D9E8',

      // White text on dark backgrounds
      // Primary text
      '--primary-text-white': '#ECF5FD',
      // Secondary text
      '--secondary-text-white': '#8DABC4',
      // Disabled text, hint text, and icons
      '--disabled-text-white': '#5b718c',
      // Dividers
      '--dividers-white': '#3F536E',

      // Gray Colors
      '--midnight': '#212F40',
      '--catalina-blue': '#2C405A',
      '--fiord': '#3F536E',
      '--nepal': '#8DABC4',
      '--light-steel-blue': '#A8C6DF',
      '--botticelli': '#C5D9E8',
      '--zumthor': '#ECF5FD',
      '--alabaster': '#FAFBFC',
      '--white': '#FFF',
      '--black': '#000',

      // Active Colors
      '--cool-blue': '#0093EE',
      '--dull-lime': '#4ECE3D',
      '--ruddy': '#FF151F',
      '--cool-blue-active': '#0079C4',
      '--dull-lime-active': '#40B630',

      // Branding Colors
      '--soft-purple': '#AB7DF6',
      '--java': '#26C2C9',
      '--supernova': '#FACA00',
      '--lime': '#81C926',
      '--pumpkin': '#FD7B1F',

      // Typography
      '--font-size': '1rem',
      '--font-bold': '600',

      // Border Radius
      '--radius-small': '2px',
      '--radius-medium': '3px',
      '--radius-large': '4px',
      '--radius-big': 'calc(var(--offset) / 2)',

      // Offset
      '--offset': '2rem',

      // Z-index
      '--ui-dropdown': '10',
      '--ui-popup': 'calc(var(--ui-dropdown) + 1)',
      '--ui-modal': 'calc(var(--ui-popup) + 1)',
      '--ui-loading': 'calc(var(--ui-modal) + 1)',
      '--ui-notification': 'calc(var(--ui-loading) + 1)',
    },

    // Sassy 高级变量
    advancedVariables: {
      unresolved: 'warn',
    },

    // 自定义 Media Queries
    media: {
      extensions: {
        '--mobile': '(width <= 425px)',
        '--over-mobile': '(width >= 426px)',
        '--into-tablet': '(width <= 768px)',
        '--tablet': '(width >= 426px) and (width <= 768px)',
        '--over-tablet': '(width >= 769px)',
        '--into-laptop': '(width <= 1024px)',
        '--laptop': '(width >= 769px) and (width <= 1024px)',
        '--over-laptop': '(width >= 1025px)',
        '--into-desktop': '(width <= 1440px)',
        '--desktop': '(width >= 1025px) and (width <= 1440px)',
        '--over-desktop': '(width >= 1441px)',
      },
      preserve: false,
      appendExtensions: false,
    },

    // postcss-assets 选项
    assets: {
      basePath: 'public/',
      loadPaths: ['assets/images/'],
    },

    // postcss-calc 选项
    calc: {
      // 计算精度
      precision: 3,
      // 是否保留原始表达式（而不是求值）
      preserve: false,
      // 是否允许在 media queries 声明里求值
      mediaQueries: true,
      // 是否允许在选择符里求值
      selectors: false,
    },

    // postcss-short 选项
    short: {
      border: {
        disable: false,
      },
      borderRadius: {
        disable: false,
      },
      color: {
        disable: false,
      },
      fontSize: {
        disable: false,
      },
      fontWeights: {
        disable: false,
      },
      position: {
        disable: false,
      },
      size: {
        disable: false,
      },
      spacing: {
        disable: false,
      },
    },

    // postcss-clear 选项
    clear: {
      display: 'block',
    },
  };

  options = merge({}, defaults, options);

  return {
    babel: {
      plugins: [['transform-object-rest-spread', { useBuiltIns: true }]],
      sourceMaps: process.env.EMBER_ENV !== 'production' ? 'inline' : false,
    },

    // 本地模块化样式选项
    cssModules: {
      // PostCSS 选项
      postcssOptions: {
        // 使用 PostCSS 生成样式的 sourcemaps，用于浏览器内对应模块的跳转（推荐）
        map: process.env.EMBER_ENV !== 'production',
      },

      // PostCSS 插件，它们的选项在本文件的前面
      plugins: {
        before: [
          require('postcss-import')(options.import),
          require('postcss-nesting'),
          require('postcss-extend'),
          require('postcss-css-variables')({
            preserve: options.preserve,
            variables: options.variables,
          }),
          require('postcss-custom-media')(options.media),
          require('postcss-media-minmax'),
          require('postcss-advanced-variables')(options.advancedVariables),
          require('postcss-calc')(options.calc),
          require('postcss-assets')(options.assets),
          require('postcss-image-set-function')({
            preserve: options.preserve,
          }),
          require('postcss-short')(options.short),
          require('postcss-clearfix')(options.clear),
          require('postcss-fallback'),
          require('postcss-color-function'),
        ],
        postprocess: [
          require('postcss-merge-rules'),
          require('postcss-discard-empty'),
          require('postcss-discard-comments'),
          require('postcss-discard-duplicates'),
        ],
      },
    },

    // 是否默认包含 Bootstrap Reboot CSS（推荐），若否，则包含 normalize.css 提供的样式
    useRebootCSS: false,

    // 是否默认包含 device.js
    useDeviceJS: false,

    // 是否默认导入 SVG 图标（'survey-client' 将导入专门为 os-client 设计的图标集），`false` 则不导入任何图标
    importSVGIcon: 'svg',

    // 是否将应用的环境选项作为 <meta> 标签注入到 HTML 文档中，否则编译到最终生成的 js 代码中
    storeConfigInMeta: process.env.EMBER_ENV === 'test',

    // 是否在产品环境下禁用 Ember Inspector
    disableInspectorInProductionMode: true,
  };
};
